/*
 MatMul.c
 ============
 
 This program demonstrates a simple matrix multiplication implementation in C,
 using two large and randomly initialized matrices.
 
 Author: S. Winberg, 2010
*/

// Standard library includes
#include <stdio.h>
#include <stdlib.h>

// Utilities for using timers
#include "timeutils.h"

// Define TESTMODULE if you want to do a multiple of a
// predefined small matrix.
//#define TESTMODULE 1

#ifdef TESTMODULE
 #define MAT_LEN 3
#else
 #define MAT_LEN 500
#endif

float random_f ()
// returns a random number within the given boundary
{ 
	return (float)rand() / RAND_MAX;
}

#ifdef TESTMODULE
void init_defmat ( float* A, float* B )
// used in checking that the matrix multiply is working
// Do this check my including the define of TESTMODULE above
{
  int i = 0;
  A[i++] = 0.19;  A[i++] = 0.892; A[i++] = 0.712;
  A[i++] = 0.896; A[i++] = 0.953; A[i++] = 0.365;
  A[i++] = 0.875; A[i++] = 0.997; A[i++] = 0.137;
  i=0;
  B[i++] = 0.19;  B[i++] = 0.892; B[i++] = 0.712;
  B[i++] = 0.896; B[i++] = 0.953; B[i++] = 0.365;
  B[i++] = 0.875; B[i++] = 0.997; B[i++] = 0.137;
}
#endif

void printmat( unsigned char* name, float* A, int n )
{
     int i,j;
     printf("%s=\n",name);
     for (j=0; j<n; j++) // rows 0..n-1
         {
         for (i=0; i<n; i++) // cols 0..n-1
            printf("%0.3f ",A[j*n+i]);
         printf("\n");
         }
}

int main ( int argc, char* argv[] )
{
    // declare variables
    unsigned long long t0,t1;
    int i,j,k;
    float sum;
    
    // calculate the CPU speed
    int cpu_speed = avg_cpu_speed(2);

    // create two vectors of MAT_LEN floats
    float* A = malloc(sizeof(float)*MAT_LEN*MAT_LEN);
    float* B = malloc(sizeof(float)*MAT_LEN*MAT_LEN);
    float* C = malloc(sizeof(float)*MAT_LEN*MAT_LEN);
    
    // create a random seed
    srand((int) time(0));
    
    // start processing
    t0 = CPU_ticks();
    
    i  = 0;
    sum = 0;
    // Do processing
    
    // first initialize the vectors
    for (j=0; j<MAT_LEN; j++)    // rows: j=0..n-1
       for (i=0; i<MAT_LEN; i++) // cols: i=0..n-1
          {
            // put random values into matrices
            A[j*MAT_LEN + i] = random_f();
            B[j*MAT_LEN + i] = random_f();
          }
    
    #ifdef TESTMODULE
     // override initialization
     init_defmat(A,B);
     printmat("A",A,MAT_LEN);
     printmat("B",A,MAT_LEN);
    #endif
    
    // perform multiplication
    for (j=0; j<MAT_LEN; j++)    // rows in A: j=0..n-1
       for (i=0; i<MAT_LEN; i++) // cols: i=0..n-1
          {
            sum = 0;
            for (k=0; k<MAT_LEN; k++) // cols in B: k=0..n-1
                sum += A[j*MAT_LEN+k] * B[k*MAT_LEN+i];
            C[j*MAT_LEN+i] = sum;
          }
          
    #ifdef TESTMODULE    
     // print result of test maltiply
     printmat("C",C,MAT_LEN);
    #endif
    
    // get the time elapsed
    t1 = CPU_ticks();
    
    // print memory budget
    unsigned msize = sizeof(float)*MAT_LEN*MAT_LEN;
    printf("Memory usage\n"
      " Matrices each: %u bytes\n Total: %u bytes (%f Megabytes)\n",
      msize,3*msize,(float)(3.0*msize)/(1024.0*1024.0)
      );
    
    // print performance results
    show_rdtsc_stats(t0,t1);
    
    system("pause");
    
    return 0;
}

