/*
 Scalarprod.c
 ============
 
 This program demonstrates a simple scalarproduct implementation in C,
 using two large and randomly initialized vectors.
 
 Author: S. Winberg, 2010
*/

// Standard library includes
#include <stdio.h>
#include <stdlib.h>

// Utilities for using timers
#include "timeutils.h"

#define VECTOR_LEN 1024

float random_f ()
//returns a random number within the given boundary
{ 
	return (float)rand() / RAND_MAX;
}

int main ( int argc, char* argv[] )
{
    // declare variables
    unsigned long long t0,t1;
    int i;
    float sum;
    
    // calculate the CPU speed
    int cpu_speed = avg_cpu_speed(2);

    // create two vectors of VECTOR_LEN floats
    float* a = malloc(sizeof(float)*VECTOR_LEN);
    float* b = malloc(sizeof(float)*VECTOR_LEN);
    
    // create a random seed
    srand((int) time(0));
    
    // start processing
    t0 = CPU_ticks();
    
    i  = 0;
    sum = 0;
    // Do processing
    
    // first initialize the vectors
    for (i=0; i<VECTOR_LEN; i++) {
          a[i] = random_f(); // get a random value and store in vector a
          b[i] = random_f(); // get a random value and store in vector b
       }
    
    for (i=0; i<VECTOR_LEN; i++) {
         sum = sum + (a[i] * b[i]);
       }
    
    // get the time elapsed
    t1 = CPU_ticks();
    
    // print results
    printf("vectorprod = %f\n",sum);
    
    // print memory budget
    unsigned msize = sizeof(float)*VECTOR_LEN;
    printf("Memory usage\n"
      " Matrices each: %u bytes\n Total: %u bytes (%f Megabytes)\n",
      msize,2*msize,(float)(2.0*msize)/(1024.0*1024.0)
      );
    
    // show perf stats
    show_rdtsc_stats(t0,t1);
    
    
    system("pause");
    
    return 0;
}

